import re

user_regex = re.compile(
	r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*\Z"  # dot-atom
	r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"\Z)',  # quoted-string
	re.IGNORECASE
)

domain_regex = re.compile(
	# max length for domain name labels is 63 characters per RFC 1034
	r'((?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+)(?:[A-Z0-9-]{2,63}(?<!-))\Z',
	re.IGNORECASE
)

def is_valid_email(email: str) -> bool:
	if '@' not in email:
		return False
	user, domain = email.rsplit('@', 1)
	return user_regex.match(user) and domain_regex.match(domain)


# -*- coding: utf-8 -*-
from .utils import is_valid_email
from .const import HeurekaService, HeurekaApiActions
from .request import HeurekaRequest

class ShopCertification:
	__slots__ = ('_api_key', '_service', '_sent', '_products_ids', '_email', '_order_id', 'response')

	def __init__(self, api_key: str, email: str, service: HeurekaService = HeurekaService.SK):
		if not is_valid_email(email):
			raise ValueError("Not valid email")
		if api_key is None or len(api_key) == 0:
			raise ValueError("Api key can't be empty")
		self._products_ids = list()
		self._email = email
		self._order_id = None
		self._sent = False
		self._api_key = api_key
		self._service = service
		self.response = None

	def add_product_id(self, product_id: str) -> None:
		if not product_id:
			raise ValueError("Product id can't be empty")
		if product_id in self._products_ids:
			raise ValueError(f"Duplicate product ids {product_id}")
		self._products_ids.append(product_id)

	@property
	def email(self) -> str:
		return self._email

	@property
	def order_id(self) -> int:
		return self._order_id

	@order_id.setter
	def order_id(self, order_id: int):
		if not str(order_id).isnumeric():
			raise TypeError("Order ID has to be integer")
		self._order_id = int(order_id)

	def log_order(self) -> bool:
		if self._sent:
			raise RuntimeError('Order can be sent just one time')
		self._sent = True
		order_data = {
			"apiKey": self._api_key,
			"email": self.email,
		}
		if self._products_ids:
			order_data['productItemIds'] = [str(i) for i in self._products_ids if i]

		if self.order_id:
			order_data['orderId'] = self.order_id

		request = HeurekaRequest(self._service)
		success, self.response = request.request(HeurekaApiActions.LOG_ORDER, get_data=None, post_data=order_data)

		if success:
			return True
		return False


import json
from urllib.parse import urlencode, urljoin

import requests

from .const import HeurekaApiActions, HeurekaService


class HeurekaRequest:
	__slots__ = ('_service')

	def __init__(self, service: HeurekaService):
		self._service = service

	def request(self, action: HeurekaApiActions, get_data: dict = None, post_data: dict = None):
		url = urljoin(self._service.get_url(), './' + action.value)
		if get_data:
			url = url + '?' + urlencode(get_data)

		if post_data:
			headers = {'Content-type': 'application/json;charset=utf-8'}
			response = requests.post(url=url, headers=headers, data=json.dumps(post_data))
		else:
			response = requests.get(url=url)
		try:
			data = response.json()
			return data['code'] == 200, response.json()
		except json.decoder.JSONDecodeError:
			status_code = response.status_code
			if status_code == 200:
				status_code = 501
			return False, {'message': response.text, 'code': status_code}



import json
from unittest import TestCase

import httpretty
import pytest
from heureka_overene import ShopCertification, HeurekaService

INVALID_INIT_PARAMETERS = [
	('', 'test@test.sk'),
	(None, 'test@test.sk'),
	('asdlfkasdf', 'test.sk'),
	('asdfasdfa', '@@test.sk')
]


@pytest.mark.parametrize('key,email', INVALID_INIT_PARAMETERS)
def test_invalid_init_paramenters(key, email):
	with pytest.raises(ValueError) as e:
		ShopCertification(key, email)


@pytest.mark.parametrize('order_id', [ 'asdf', '23b', None, ])
def test_invalid_order_ids(order_id):
	shop_certification = ShopCertification('adsfasdfasdf', 'test@test.sk')
	with pytest.raises(TypeError) as e:
		shop_certification.order_id = order_id


@pytest.mark.parametrize('email',['info@google.com', 'info@test.local'])
def test_valid_init_email(email):
	ShopCertification('adsfasdfasdf', email)


@httpretty.activate(allow_net_connect=False)
def test_send_base():
	api_url = 'https://api.heureka.cz/shop-certification/v2/order/log'
	httpretty.register_uri(httpretty.POST, api_url, status=200, body='{"code": 200, "message": "ok"}')
	shop_certification = ShopCertification('adsfasdfasdf', 'info@info.sk', service=HeurekaService.CZ)
	assert True == shop_certification.log_order()
	last_request = httpretty.last_request()
	assert 'Content-type' in last_request.headers
	assert 'application/json' in last_request.headers['Content-type']


@httpretty.activate(allow_net_connect=False)
def test_invalid_auth():
	api_url = 'https://api.heureka.cz/shop-certification/v2/order/log'
	response_data = '{ "code": 401, "message": "unauthorized", "description": "Unknown API key" }'
	httpretty.register_uri(httpretty.POST, api_url, status=401, body=response_data)
	shop_certification = ShopCertification('adsfasdfasdf', 'info@info.sk', service=HeurekaService.CZ)
	assert False == shop_certification.log_order()


def test_invalid_product_id():
	certification = ShopCertification('alskdjfasdf', 'test@test.sk')
	with pytest.raises(ValueError):
		certification.add_product_id('')
	with pytest.raises(ValueError):
		certification.add_product_id(None)
	certification.add_product_id('PORD-12')
	with pytest.raises(ValueError):
		certification.add_product_id('PORD-12')


@httpretty.activate(allow_net_connect=False, verbose=True)
def test_send_full_data():
	httpretty.register_uri(httpretty.POST, 'https://api.heureka.sk/shop-certification/v2/order/log', responses=200, body='{"code": 200, "message": "ok"}')

	API_KEY = 'asdflaksdfasdf'
	EMIAL = 'test@test.sk'
	ORDER_ID = 2342
	PRODUCTS_IDS = ['123', '234234', 'ad23423']
	shop_certification = ShopCertification(API_KEY, EMIAL)
	for pk in PRODUCTS_IDS:
		shop_certification.add_product_id(pk)
	shop_certification.order_id = ORDER_ID
	assert True == shop_certification.log_order()
	last_request = httpretty.last_request()
	TestCase().assertDictEqual({
		"apiKey": API_KEY,
		"email": EMIAL,
		"orderId": ORDER_ID,
		"productItemIds": PRODUCTS_IDS
	}, json.loads(last_request.body))

	with pytest.raises(RuntimeError):
		shop_certification.log_order()



# -*- coding: utf-8 -*-
from heureka_overene.request import HeurekaRequest
from heureka_overene import HeurekaApiActions, HeurekaService
import httpretty
import pytest
import json


REQUEST_URL_PARAMETERS = [
	('https://api.heureka.cz/shop-certification/v2/order/log', httpretty.GET, HeurekaService.CZ, {}, {}),
	('https://api.heureka.sk/shop-certification/v2/order/log', httpretty.GET, HeurekaService.SK, {}, {}),
	('https://api.heureka.sk/shop-certification/v2/order/log?test=fast', httpretty.GET, HeurekaService.SK, {'test': 'fast'}, {}),
	('https://api.heureka.sk/shop-certification/v2/order/log?test=fast&params=val', httpretty.GET, HeurekaService.SK, {'test': 'fast', 'params': 'val'}, {}),
	('https://api.heureka.cz/shop-certification/v2/order/log', httpretty.POST, HeurekaService.CZ, {}, {'jano': 10}),
	('https://api.heureka.sk/shop-certification/v2/order/log', httpretty.POST, HeurekaService.SK, {}, {'data': 'valid'}),
	('https://api.heureka.sk/shop-certification/v2/order/log?test=fast', httpretty.POST, HeurekaService.SK, {'test': 'fast'}, {'data': 'valid'}),
	('https://api.heureka.sk/shop-certification/v2/order/log?test=fast&params=val', httpretty.POST, HeurekaService.SK, {'test': 'fast', 'params': 'val'}, {'data': 'valid'}),
]


@httpretty.activate(allow_net_connect=False, verbose=True)
@pytest.mark.parametrize('url, http_method, service, get_data, post_data', REQUEST_URL_PARAMETERS)
def test_request_url(url, http_method, service, get_data, post_data):
	request = HeurekaRequest(service)
	httpretty.register_uri(http_method, url, responses=200, body='{"code": 200, "message": "ok"}')
	success, data = request.request(HeurekaApiActions.LOG_ORDER, get_data=get_data, post_data=post_data)
	assert success
	assert data['code'] == 200
	last_request = httpretty.last_request()
	assert last_request.url == url
	assert last_request.method == http_method


@httpretty.activate(allow_net_connect=False, verbose=True)
def test_request_miss_match_codes():
	api_url = 'https://api.heureka.cz/shop-certification/v2/order/log'
	response_data = json.dumps({ "code": 401, "message": "unauthorized", "description": "Unknown API key \"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\"." })
	request = HeurekaRequest(HeurekaService.CZ)
	httpretty.register_uri(httpretty.POST, api_url, status=301, body=response_data)
	success, data = request.request(HeurekaApiActions.LOG_ORDER, get_data={}, post_data={'key': 'val'})
	assert success == False
	assert data['code'] == 401
	last_request = httpretty.last_request()

   

@httpretty.activate(allow_net_connect=False, verbose=True)
def test_request_server_error():
	api_url = 'https://api.heureka.cz/shop-certification/v2/order/log'
	request = HeurekaRequest(HeurekaService.CZ)
	httpretty.register_uri(httpretty.POST, api_url, status=500, body='Unknown server error')
	success, data = request.request(HeurekaApiActions.LOG_ORDER, get_data={}, post_data={'key': 'val'})
	assert success == False
	assert data['code'] == 500


import enum


class HeurekaService(enum.Enum):
	SK = 'https://api.heureka.sk/shop-certification/v2/'
	CZ = 'https://api.heureka.cz/shop-certification/v2/'

	def get_url(self):
		return self.value


class HeurekaApiActions(enum.Enum):
	LOG_ORDER = 'order/log'



# -*- coding: utf-8 -*-
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
	long_description = fh.read()

setuptools.setup(
	name="heureka_overene",
	version="0.0.3",
	author="Michal Kubek",
	author_email="kubek@wisdomtech.sk",
	description="Heureka API client for service overene zákazníkmi",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/wisdom-technologies/heureka-overene",
	project_urls={
		"Bug Tracker": "https://github.com/wisdom-technologies/heureka-overene/issues",
	},
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	package_dir={"": "src"},
	packages=setuptools.find_packages(where="src"),
	python_requires=">=3.6",
	install_requires=[
		'requests',
	],
	extras_require={
		'test': [
			'httpretty',
			'pytest'
		]
	}
)
